<?php
/**
 * 京东数据接口
 * @作者 苏相锟 <admin@phpok.com>
 * @主页 https://www.phpok.com
 * @版本 5.x
 * @授权 GNU Lesser General Public License https://www.phpok.com/lgpl.html 
 * @时间 2021年2月26日
**/

/**
 * 安全限制，防止直接访问
**/
if(!defined("PHPOK_SET")){
	exit("<h1>Access Denied</h1>");
}

class jd_lib extends _init_lib
{
	private $client_id = '';
	private $client_secret = '';
	private $client_user = '';
	private $client_pass = '';
	private $buyer_nick = ''; // 用户昵称
	private $session_key = '';
	private $version = '1.0';
	private $sign_method = 'MD5';
	private $timestamp = '';
	private $method = '';
	private $curl;
	private $url='https://bizapi.jd.com/';
	private $systime = 0;//系统当前时间，这里读的是时间戳
	public function __construct()
	{
		parent::__construct();
		$this->curl = $GLOBALS['app']->lib('curl');
		$this->timestamp = date("Y-m-d H:i:s",time());
		$this->systime = time();
	}

	public function access_token()
	{
		$file = $this->dir_data."jd_access_token.php";
		if(file_exists($file) && (filemtime($file)-72000)>$this->time){
			include($file);
			return $config['access_token'];
		}
		$data = array();
		$data['grant_type'] = 'access_token';
		$data['client_id'] = $this->client_id;
		$data['timestamp'] = $this->timestamp;
		$data['username'] = $this->client_user;
		$data['password'] = $this->client_pass;
		$data['sign'] = md5($this->client_secret.$data['timestamp'].$data['client_id'].$data['username'].$data['password'].$data['grant_type'].$this->client_secret);
		$data['sign'] = strtoupper($data['sign']);
		$url = $this->url.'oauth2/accessToken';
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$rs = $this->curl->get_json($url);
		if(!$rs['success']){
			$this->log($rs['resultMessage'],$rs['resultCode']);
			return false;
		}
		$this->lib('file')->vi($rs['result'],$file,'config');
		return $rs['result']['access_token'];
	}

	public function client_id($val='')
	{
		if($val){
			$this->client_id = $val;
		}
		return $this->client_id;
	}

	public function client_secret($val='')
	{
		if($val){
			$this->client_secret = $val;
		}
		return $this->client_secret;
	}

	public function client_user($val='')
	{
		if($val){
			$this->client_user = $val;
		}
		return $this->client_user;
	}

	public function client_pass($val='')
	{
		if($val){
			$this->client_pass = md5($val);
		}
		return $this->client_pass;
	}

	public function log($info,$code='')
	{
		if(!$info){
			return false;
		}
		$info = trim($info);
		$date = date("Ymd",$this->time);
		if(!file_exists($this->dir_data.'log/log'.$date.'.php')){
			file_put_contents($this->dir_data.'log/log'.$date.'.php',"<?php exit();?>\n");
		}
		$handle = fopen($this->dir_data.'log/log'.$date.'.php','ab');
		$info2 = '---start---Time:'.date("H:i:s",$this->time).'---------------------'."\n";
		if($code){
			$info2 .= $code."\n";
		}
		$info2.= $info."\n";
		$info2.= '---end---'."\n";
		fwrite($handle,$info2);
		fclose($handle);
		return true;
	}

	public function method($val='')
	{
		if($val){
			$this->method = $val;
		}
		return $this->method;
	}

	/**
	 * 订单取消
	 * @参数 $sn 要取消的订单
	**/
	public function order_cancel($sn='')
	{
		$this->method('epet.trade.order.cancel');
		$data = array();
		$data['trade_no'] = $sn;
		$postdata = $this->postdata($data);
		$sign = $this->sign($postdata);
		$postdata['sign'] = $sign;
		$poststr = json_encode($postdata);
		$this->curl->post_data($poststr);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/json; charset=utf-8');
		return $this->curl->get_json($this->url);
	}

	/**
	 * 创建订单
	 * @参数 $order 订单系统
	 * @参数 $prolist 产品列表
	 * @参数 $address 收件人信息
	 * @参数 $fee 快递费，为0表示免邮费
	**/
	public function order_create($order,$prolist,$address,$fee=0)
	{
		if(!$order || !$prolist || !$address){
			return false;
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$pro = array();
		foreach($prolist as $key=>$value){
			$tmp = array();
			$tmp['skuId'] = $value['jdpro_id'];
			$tmp['num'] = $value['qty'];
			$tmp['price'] = $value['price'];
			$tmp['bNeedGift'] = false;
			$pro[] = $value;
		}
		
		$data = array('token'=>$access_token);
		$data['thirdOrder'] = $order['sn'];
		$data['sku'] = json_encode($pro,JSON_UNESCAPED_UNICODE);
		$data['name'] = $address['fullname'];
		$data['province'] = $address['province_id'];
		$data['city'] = $address['city_id'];
		$data['county'] = $address['county_id'];
		$data['town'] = '0';
		$data['address'] = $address['address'];
		$data['mobile'] = $address['mobile'] ? $address['mobile'] : $order['mobile'];
		$data['email'] = $address['email'] ? $address['email'] : $order['email'];
		if(!$data['email']){
			$data['email'] = "admin@phpok.com";
		}
		$data['invoiceState'] = 4;//支付方式（1在线支付；2货到付款；4三方平台代收）
		$data['invoiceType'] = 3;
		$data['selectedInvoiceTitle'] = 4;
		$data['invoiceContent'] = 100;
		$data['paymentType'] = 101;
		$data['isUseBalance'] = 0;
		$data['submitState'] = 0;
		$data['invoicePhone'] = $data['mobile'];

		$url = $this->url.'api/order/submitOrder';
		$data = array('token'=>$access_token);
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		$rs = $this->curl->get_json($url);
		if(!$rs['success']){
			$this->log($rs['resultMessage'],$rs['resultCode']);
			return false;
		}
		return $rs;
	}

	public function order_finish($sn='')
	{
		$this->method('epet.trade.order.finish');
		$data = array();
		$data['trade_no'] = $sn;
		$postdata = $this->postdata($data);
		$sign = $this->sign($postdata);
		$postdata['sign'] = $sign;
		$poststr = json_encode($postdata);
		$this->curl->post_data($poststr);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/json; charset=utf-8');
		return $this->curl->get_json($this->url);
	}

	/**
	 * 订单批量查询
	 * @参数 $sn 支付字符串或数组 
	**/
	public function order_query($sn='')
	{
		if(!$sn){
			return false;
		}
		if(!is_array($sn)){
			$sn = array($sn);
		}
		$this->method('epet.trade.order.batch.get');
		$data = array();
		$data['trade_nos'] = $sn;
		$postdata = $this->postdata($data);
		$sign = $this->sign($postdata);
		$postdata['sign'] = $sign;
		$poststr = json_encode($postdata);
		$this->curl->post_data($poststr);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/json; charset=utf-8');
		return $this->curl->get_json($this->url);
	}

	/**
	 * 更新订单
	 * @参数 $order 订单系统
	 * @参数 $prolist 产品列表
	 * @参数 $address 收件人信息
	 * @参数 $fee 快递费，为0表示免邮费
	**/
	public function order_update($order,$prolist,$address,$fee=0)
	{
		if(!$order || !$prolist || !$address){
			return false;
		}
		$this->method('epet.trade.order.update');
		$data = array();
		$data['trade_no'] = $order['sn'];
		$data['buyer_nick'] = 'Epet-'.$order['user_id'];
		$data['payment'] = round($order['price'],2);
		$data['total_price'] = round($order['price'],2);
		$data['post_fee'] = $fee;
		$data['pay_type'] = 4;//支付方式（1在线支付；2货到付款；4三方平台代收）
		$data['receiver'] = array();
		$data['receiver']['receiver_name'] = $address['fullname'];
		$data['receiver']['receiver_province'] = $address['province'];
		$data['receiver']['receiver_city'] = $address['city'];
		$data['receiver']['receiver_area'] = $address['county'];
		$data['receiver']['receiver_mobile'] = $address['mobile'] ? $address['mobile'] : $address['tel'];
		$data['receiver']['receiver_address'] = $address['address'].$address['address2'];
		if($order['note']){
			$data['remark'] = $order['note'];
		}
		$data['buy_time'] = date("Y-m-d H:i:s",$order['addtime']);
		$data['pay_time'] = date("Y-m-d H:i:s",$order['paytime']);
		$data['goods'] = array();
		foreach($prolist as $key=>$value){
			$tmp = array();
			$tmp['outer_sku'] = $value['goods_gid'];
			$tmp['num'] = $value['qty'];
			$tmp['title'] = $value['title'];
			$tmp['price'] = round($value['price'],2);
			$data['goods'][] = $tmp;
		}
		$postdata = $this->postdata($data);
		$sign = $this->sign($postdata);
		$postdata['sign'] = $sign;
		$poststr = json_encode($postdata);
		$this->curl->post_data($poststr);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/json; charset=utf-8');
		return $this->curl->get_json($this->url);
	}


	public function price($pin='')
	{
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$data = array('token'=>$access_token);
		$data['pin'] = $pin;
		$data['type'] = "1,2";
		$url = $this->url."api/price/getUnionBalance";
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		return $this->curl->get_json($url);
	}


	public function postdata($data='')
	{
		$tlist = array();
		$tlist['app_id'] = (string) $this->app_id;
		$tlist['session_key'] = $this->session_key;
		$tlist['method'] = $this->method;
		$tlist['timestamp'] = $this->timestamp;
		$tlist['version'] = $this->version;
		$tlist['format'] = 'json';
		$tlist['sign_method'] = $this->sign_method;
		if($data && is_array($data)){
			ksort($data);
			$tlist['body'] = json_encode($data);
		}else{
			if($data){
				$tlist['body'] = $data;
			}
		}
		ksort($tlist);
		return $tlist;
	}

	public function product_catelist($update=false)
	{
		$file = $this->dir_data.'jd_catelist.php';
		if(!$update && file_exists($file)){
			include $file;
			return $rslist;
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$url = $this->url.'api/product/getPageNum';
		$data = array('token'=>$access_token);
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		$rs = $this->curl->get_json($url);
		if(!$rs['success']){
			$this->log($rs['resultMessage'],$rs['resultCode']);
			return false;
		}
		$rslist = $rs['result'];
		$this->lib('file')->vi($rslist,$file,'rslist');
		return $rslist;
	}


	public function product_list($condition='',$pageid=0,$psize=20)
	{
		if(!$end){
			$end = $this->systime;
		}
		if(!$start){
			$start = $end - 365*24*60*60;
		}
		if(!$pageid){
			$pageid = 1;
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$data = array('token'=>$access_token);
		if($condition && is_array($condition)){
			if($condition['keywords']){
				$data['Keyword'] = $condition['keywords'];
			}
			if($condition['cateid']){
				$data['catId'] = $condition['cateid'];
			}
			if($condition['min_price']){
				$data['Min'] = $condition['min_price'];
			}
			if($condition['max_price']){
				$data['Max'] = $condition['max_price'];
			}
			if($condition['brands']){
				$data['Brands'] = $condition['brands'];
			}
		}
		$data['priceCol'] = 'yes';
		$data['redisStore'] = 'true';
		$data['pageIndex'] = $pageid;
		$data['pageSize'] = $psize;
		$url = $this->url."api/search/search";
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		return $this->curl->get_json($url);
	}

	public function product_images($sku='')
	{
		if(!$sku){
			return false;
		}
		if(is_array($sku)){
			$sku = implode(",",$sku);
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$data = array('token'=>$access_token);
		$data['sku'] = $sku;
		$url = $this->url."api/product/skuImage";
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		return $this->curl->get_json($url);
	}

	public function product_state($sku='')
	{
		if(!$sku){
			return false;
		}
		if(is_array($sku)){
			$sku = implode(",",$sku);
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$data = array('token'=>$access_token);
		$data['sku'] = $sku;
		$url = $this->url."api/product/skuState";
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		return $this->curl->get_json($url);
	}

	public function product_check($sku='')
	{
		if(!$sku){
			return false;
		}
		if(is_array($sku)){
			$sku = implode(",",$sku);
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$data = array('token'=>$access_token);
		$data['skuIds'] = $sku;
		$url = $this->url."api/product/check";
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		return $this->curl->get_json($url);
	}

	/**
	 * 查看商品价格
	 * @参数 $sku 商品编号，请以，(英文逗号)分割。例如：129408,129409 (最高支持100个商品)
	**/
	public function product_price($sku='')
	{
		if(!$sku){
			return false;
		}
		if(is_array($sku)){
			$sku = implode(",",$sku);
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$data = array('token'=>$access_token);
		$data['sku'] = $sku;
		$url = $this->url."api/price/getSellPrice";
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		return $this->curl->get_json($url);
	}

	public function product_ids($id=0,$pageid=1)
	{
		if(!$id){
			return false;
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$data = array('token'=>$access_token);
		$data['pageNum'] = $id;
		$data['pageNo'] = $pageid;
		$url = $this->url."api/product/getSkuByPage";
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		return $this->curl->get_json($url);
	}

	public function product_info($id=0)
	{
		if(!$id){
			return false;
		}
		$access_token = $this->access_token();
		if(!$access_token){
			$this->log('获取 accessToken 失败，请检查');
			return false;
		}
		$data = array('token'=>$access_token);
		$data['sku'] = $id;
		$url = $this->url."api/product/getDetail";
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		return $this->curl->get_json($url);
	}

	public function product_stock($id=0)
	{
		if(!$id){
			return false;
		}
		if($id && !is_array($id)){
			$tmp = array($id);
			$id = $tmp;
			unset($tmp);
		}
		$this->method('epet.stock.get');
		$data = array('goods_gids'=>$id);
		$postdata = $this->postdata($data);
		$sign = $this->sign($postdata);
		$postdata['sign'] = $sign;
		$poststr = json_encode($postdata);
		$this->curl->post_data($poststr);
		$this->curl->is_post(true);
		$this->curl->set_header('Content-Type','application/json; charset=utf-8');
		return $this->curl->get_json($this->url);
	}

	public function refresh_token()
	{
		$file = $this->dir_data."jd_access_token.php";
		if(!file_exists($file) || (filemtime($file)-72000)<$this->time){
			return true;
		}
		include($file);
		$data = array('refresh_token'=>$config['refresh_token']);
		$data['client_id'] = $this->client_id;
		$data['client_secret'] = $this->client_secret;
		$url = $this->url.'oauth2/refreshToken';
		$this->curl->set_header('Content-Type','application/x-www-form-urlencoded');
		$this->curl->post_data($data);
		$this->curl->is_post(true);
		$rs = $this->curl->json($url);
		if(!$rs['success']){
			$this->log($rs['resultMessage'],$rs['resultCode']);
			return false;
		}
		$this->lib('file')->vi($rs['result'],$file,'config');
		return true;
	}

	public function session_key($val='')
	{
		if($val){
			$this->session_key = $val;
		}
		return $this->session_key;
	}

	public function systime($val='')
	{
		if($val){
			$this->systime = $val;
		}
		return $this->systime;
	}

	public function sign($data = array())
	{
		$string = $this->app_secret;
		ksort($data);
		foreach($data as $key=>$value){
			if($value == '' || $key == 'sign'){
				continue;
			}
			$string .= $key.''.$value;
		}
		$string .= $this->app_secret;
		$code = md5($string);
		return strtoupper($code);
	}

	public function timestamp($val='')
	{
		if($val && is_numeric($val)){
			$val = date("Y-m-d H:i:s",$val);
		}
		if($val){
			$this->timestamp = $val;
		}
		return $this->timestamp;
	}

	public function url($val='')
	{
		if($val){
			$this->url = $val;
		}
		return $this->url;
	}
}
